from __future__ import (print_function, division)

import unittest
import os

from mock import mock
from WMCore.Storage.Backends.XRDCPImpl import XRDCPImpl


class XRDCPImplTest(unittest.TestCase):
    def setUp(self):
        self.XRDCPImpl = XRDCPImpl()

    def testCreateSourceName_simple(self):
        self.assertEqual("name", self.XRDCPImpl.createSourceName("", "name"))

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.splitPFN')
    def testCreateStageOutCommand_noStageInNoOptionsNoChecksum(self, mock_splitPFN):
        mock_splitPFN.return_value = (None, "host", "path", None)
        self.XRDCPImpl.stageIn = False
        results = self.XRDCPImpl.createStageOutCommand("sourcePFN", "targetPFN")
        expectedResults = self.createStageOutCommandResults(False, "sourcePFN", "targetPFN", "sourcePFN", None, False,
                                                            False, "host", "path")
        self.assertEqual(expectedResults, results)

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.splitPFN')
    def testCreateStageOutCommand_stageInNoOptionsNoChecksum(self, mock_splitPFN):
        mock_splitPFN.return_value = (None, "host", "path", None)
        self.XRDCPImpl.stageIn = True
        results = self.XRDCPImpl.createStageOutCommand("sourcePFN", "targetPFN")
        expectedResults = self.createStageOutCommandResults(True, "sourcePFN", "targetPFN", "targetPFN", None, False,
                                                            False, "host", "path")
        self.assertEqual(expectedResults, results)

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.splitPFN')
    def testCreateStageOutCommand_optionsNoStageInNoChecksum(self, mock_splitPFN):
        mock_splitPFN.return_value = (None, "host", "path", None)
        self.XRDCPImpl.stageIn = False
        results = self.XRDCPImpl.createStageOutCommand("sourcePFN", "targetPFN", options="--wma-cerncastor")
        expectedResults = self.createStageOutCommandResults(False, "sourcePFN", "targetPFN", "sourcePFN",
                                                            "--wma-cerncastor", False, False, "host", "path")
        self.assertEqual(expectedResults, results)

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.splitPFN')
    def testCreateStageOutCommand_optionsUnknowsNoStageInNoChecksum(self, mock_splitPFN):
        mock_splitPFN.return_value = (None, "host", "path", None)
        self.XRDCPImpl.stageIn = False
        results = self.XRDCPImpl.createStageOutCommand("sourcePFN", "targetPFN", options="--wma-cerncastor --test")
        expectedResults = self.createStageOutCommandResults(False, "sourcePFN", "targetPFN", "sourcePFN",
                                                            "--wma-cerncastor", "--test", False, "host", "path")
        self.assertEqual(expectedResults, results)

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.splitPFN')
    def testCreateStageOutCommand_optionsUnknowsStageInNoChecksum(self, mock_splitPFN):
        mock_splitPFN.return_value = (None, "host", "path", None)
        self.XRDCPImpl.stageIn = True
        results = self.XRDCPImpl.createStageOutCommand("sourcePFN", "targetPFN", options="--wma-cerncastor --test")
        expectedResults = self.createStageOutCommandResults(True, "sourcePFN", "targetPFN", "targetPFN", "--wma-cerncastor",
                                                            "--test", False, "host", "path")
        self.assertEqual(expectedResults, results)

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.splitPFN')
    def testCreateStageOutCommand_optionsUnknowsStageInChecksum(self, mock_splitPFN):
        mock_splitPFN.return_value = (None, "host", "path", None)
        self.XRDCPImpl.stageIn = True
        results = self.XRDCPImpl.createStageOutCommand("sourcePFN", "targetPFN", options="--wma-cerncastor --test",
                                                       checksums=["adler32"])
        expectedResults = self.createStageOutCommandResults(True, "sourcePFN", "targetPFN", "targetPFN", "--wma-cerncastor",
                                                            "--test", False, "host", "path")
        self.assertEqual(expectedResults, results)

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.splitPFN')
    def testCreateStageOutCommand_optionsNoStageInChecksum(self, mock_splitPFN):
        mock_splitPFN.return_value = (None, "host", "path", None)
        self.XRDCPImpl.stageIn = False
        results = self.XRDCPImpl.createStageOutCommand("sourcePFN", "targetPFN", options="--wma-cerncastor --test",
                                                       checksums={'adler32': "32"})
        expectedResults = self.createStageOutCommandResults(False, "sourcePFN", "targetPFN", "sourcePFN",
                                                            "--wma-cerncastor", "--test", "00000032", "host", "path")
        self.assertEqual(expectedResults, results)

    def createStageOutCommandResults(self, stageIn, sourcePFN, targetPFN, localPFN, copyCommandOptions, unknow,
                                     checksums, host, path):
        copyCommand = ""
        if not stageIn:
            copyCommand += "LOCAL_SIZE=`stat -c%%s \"%s\"`\n" % localPFN
            copyCommand += "echo \"Local File Size is: $LOCAL_SIZE\"\n"
            if copyCommandOptions:
                targetPFN += "?svcClass=t0cms"
        initFile = "/cvmfs/oasis.opensciencegrid.org/osg-software/osg-wn-client/current/el7-x86_64/setup.sh"
        if not self.XRDCPImpl._checkXRDUtilsExist() and os.path.isfile(initFile):
            copyCommand += "source {}\n".format(initFile)
        copyCommand += "xrdcp --force --nopbar "
        if unknow:
            copyCommand += "%s " % unknow
        if checksums:
            copyCommand += "--cksum adler32:%s " % checksums
        copyCommand += " \"%s\" " % sourcePFN
        copyCommand += " \"%s\" \n" % targetPFN
        if stageIn:
            copyCommand += "LOCAL_SIZE=`stat -c%%s \"%s\"`\n" % localPFN
            copyCommand += "echo \"Local File Size is: $LOCAL_SIZE\"\n"
            removeCommand = ""
        else:
            removeCommand = "xrdfs '%s' rm %s ;" % (host, path)
        copyCommand += "REMOTE_SIZE=`xrdfs '%s' stat '%s' | grep Size | sed -r 's/.*Size:[ ]*([0-9]+).*/\\1/'`\n" % (
            host, path)
        copyCommand += "echo \"Remote File Size is: $REMOTE_SIZE\"\n"
        if checksums:
            copyCommand += "echo \"Local File Checksum is: %s\"\n" % checksums
            copyCommand += "REMOTE_XS=`xrdfs '%s' query checksum '%s' | grep -i adler32 | sed -r 's/.*[adler|ADLER]32[ ]*([0-9a-fA-F]{8}).*/\\1/'`\n" % (
                host, path)
            copyCommand += "echo \"Remote File Checksum is: $REMOTE_XS\"\n"

            copyCommand += "if [ $REMOTE_SIZE ] && [ $REMOTE_XS ] && [ $LOCAL_SIZE == $REMOTE_SIZE ] && [ '%s' == $REMOTE_XS ]; then exit 0; " % \
                           checksums
            copyCommand += "else echo \"ERROR: Size or Checksum Mismatch between local and SE\"; %s exit 60311 ; fi" % removeCommand
        else:
            copyCommand += "if [ $REMOTE_SIZE ] && [ $LOCAL_SIZE == $REMOTE_SIZE ]; then exit 0; "
            copyCommand += "else echo \"ERROR: Size Mismatch between local and SE\"; %s exit 60311 ; fi" % removeCommand
        return copyCommand

    @mock.patch('WMCore.Storage.Backends.XRDCPImpl.XRDCPImpl.executeCommand')
    def testRemoveFile(self, mock_executeCommand):
        self.XRDCPImpl.removeFile("gsiftp://site.com/inputs/f.a")
        mock_executeCommand.assert_called_with("xrdfs site.com rm inputs/f.a")



#!/usr/bin/env python

"""
BossAir preliminary test
"""
from __future__ import print_function

import getpass
import os.path
import subprocess
import threading
import unittest
import pickle

from nose.plugins.attrib import attr

import WMCore.WMInit
from WMCore.DAOFactory import DAOFactory
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMSpec.Makers.TaskMaker import TaskMaker
from WMCore.JobStateMachine.ChangeState import ChangeState
from WMCore.ResourceControl.ResourceControl import ResourceControl
from WMCore.Agent.HeartbeatAPI import HeartbeatAPI
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Workflow import Workflow
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Job import Job
from WMCore.BossAir.BossAirAPI import BossAirAPI, BossAirException

from WMCore_t.WMSpec_t.TestSpec import testWorkload
from WMQuality.TestInitCouchApp import TestInitCouchApp as TestInit


def getNArcJobs():
    """
    Return the number of active ARC jobs.

    """

    cmd = "wc -l ~/.ngjobs"
    pipe = subprocess.Popen(cmd, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, shell=True)
    stdout, _ = pipe.communicate()
    nJobs = int(stdout.split()[0])
    return nJobs


def getCondorRunningJobs(user):
    """
    _getCondorRunningJobs_

    Return the number of jobs currently running for a user
    """

    command = ['condor_q', user]
    pipe = subprocess.Popen(command, stdout=subprocess.PIPE,
                            stderr=subprocess.PIPE, shell=False)
    stdout, _ = pipe.communicate()

    output = stdout.split('\n')[-2]

    nJobs = int(output.split(';')[0].split()[0])

    return nJobs


class BossAirTest(unittest.TestCase):
    """
    Tests for the BossAir prototype

    """

    sites = ['T2_US_UCSD', 'T2_TW_Taiwan', 'T1_CH_CERN', 'T2_US_Florida']

    def setUp(self):
        """
        setup for test.
        """

        myThread = threading.currentThread()

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection(destroyAllDatabase=True)
        self.tearDown()
        self.testInit.setSchema(
            customModules=["WMCore.WMBS", "WMCore.BossAir", "WMCore.ResourceControl", "WMCore.Agent.Database"],
            useDefault=False)
        self.testInit.setupCouch("bossair_t/jobs", "JobDump")
        self.testInit.setupCouch("bossair_t/fwjrs", "FWJRDump")

        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)
        self.getJobs = self.daoFactory(classname="Jobs.GetAllJobs")

        # Create sites in resourceControl
        resourceControl = ResourceControl()
        for site in self.sites:
            resourceControl.insertSite(siteName=site, pnn='%s_PNN' % site, cmsName=site,
                                       ceName=site, plugin="SimpleCondorPlugin", pendingSlots=1000,
                                       runningSlots=2000)
            resourceControl.insertThreshold(siteName=site, taskType='Processing',
                                            maxSlots=1000, pendingSlots=1000)

        site = 'T3_US_Xanadu'
        resourceControl.insertSite(siteName=site, pnn='%s_PNN' % site, cmsName=site,
                                   ceName=site, plugin="TestPlugin")
        resourceControl.insertThreshold(siteName=site, taskType='Processing',
                                        maxSlots=10000, pendingSlots=10000)

        # Create user
        newuser = self.daoFactory(classname="Users.New")
        newuser.execute(dn="tapas", group_name="phgroup", role_name="cmsrole")

        # We actually need the user name
        self.user = getpass.getuser()

        # Change this to the working dir to keep track of error and log files from condor
        self.testDir = self.testInit.generateWorkDir()

        # Set heartbeat
        componentName = 'test'
        self.heartbeatAPI = HeartbeatAPI(componentName)
        self.heartbeatAPI.registerComponent()
        componentName = 'JobTracker'
        self.heartbeatAPI2 = HeartbeatAPI(componentName)
        self.heartbeatAPI2.registerComponent()

        return

    def tearDown(self):
        """
        Database deletion
        """
        # self.testInit.clearDatabase(modules = ["WMCore.WMBS", "WMCore.BossAir", "WMCore.ResourceControl", "WMCore.Agent.Database"])

        self.testInit.delWorkDir()

        self.testInit.tearDownCouch()

        return

    def getConfig(self):
        """
        _getConfig_

        Build a basic BossAir config
        """

        config = self.testInit.getConfiguration()

        config.section_("Agent")
        config.Agent.agentName = 'testAgent'
        config.Agent.componentName = 'test'
        config.Agent.useHeartbeat = False
        config.Agent.isDocker = False

        config.section_("CoreDatabase")
        config.CoreDatabase.connectUrl = os.getenv("DATABASE")
        config.CoreDatabase.socket = os.getenv("DBSOCK")

        config.section_("BossAir")
        config.BossAir.pluginNames = ['TestPlugin', 'SimpleCondorPlugin']
        config.BossAir.pluginDir = 'WMCore.BossAir.Plugins'
        config.BossAir.UISetupScript = '/afs/cern.ch/cms/LCG/LCG-2/UI/cms_ui_env.sh'

        config.component_("JobSubmitter")
        config.JobSubmitter.logLevel = 'INFO'
        config.JobSubmitter.pollInterval = 1
        config.JobSubmitter.pluginName = 'SimpleCondorPlugin'
        config.JobSubmitter.pluginDir = 'JobSubmitter.Plugins'
        config.JobSubmitter.submitDir = os.path.join(self.testDir, 'submit')
        config.JobSubmitter.submitNode = os.getenv("HOSTNAME", 'stevia.hep.wisc.edu')
        config.JobSubmitter.submitScript = os.path.join(WMCore.WMInit.getWMBASE(),
                                                        'test/python/WMComponent_t/JobSubmitter_t',
                                                        'submit.sh')
        config.JobSubmitter.componentDir = os.path.join(os.getcwd(), 'Components')
        config.JobSubmitter.workerThreads = 2
        config.JobSubmitter.jobsPerWorker = 200
        config.JobSubmitter.gLiteConf = os.path.join(os.getcwd(), 'config.cfg')

        # JobTracker
        config.component_("JobTracker")
        config.JobTracker.logLevel = 'INFO'
        config.JobTracker.pollInterval = 1

        # JobStateMachine
        config.component_('JobStateMachine')
        config.JobStateMachine.couchurl = os.getenv('COUCHURL')
        config.JobStateMachine.couchDBName = "bossair_t"

        # JobStatusLite
        config.component_('JobStatusLite')
        config.JobStatusLite.componentDir = os.path.join(os.getcwd(), 'Components')
        config.JobStatusLite.stateTimeouts = {'Pending': 10, 'Running': 86400}
        config.JobStatusLite.pollInterval = 1

        return config

    def createTestWorkload(self, workloadName='Test', emulator=True):
        """
        _createTestWorkload_

        Creates a test workload for us to run on, hold the basic necessities.
        """

        workload = testWorkload("Tier1ReReco")
        rereco = workload.getTask("ReReco")

        taskMaker = TaskMaker(workload, os.path.join(self.testDir, 'workloadTest'))
        taskMaker.skipSubscription = True
        taskMaker.processWorkload()

        workload.save(workloadName)

        return workload

    def createJobGroups(self, nSubs, nJobs, task, workloadSpec, site=None, bl=[], wl=[]):
        """
        Creates a series of jobGroups for submissions

        """

        jobGroupList = []

        testWorkflow = Workflow(spec=workloadSpec, owner="tapas",
                                name=makeUUID(), task="basicWorkload/Production",
                                owner_vogroup='phgroup', owner_vorole='cmsrole')
        testWorkflow.create()

        # Create subscriptions
        for i in range(nSubs):
            name = makeUUID()

            # Create Fileset, Subscription, jobGroup
            testFileset = Fileset(name=name)
            testFileset.create()
            testSubscription = Subscription(fileset=testFileset,
                                            workflow=testWorkflow,
                                            type="Processing",
                                            split_algo="FileBased")
            testSubscription.create()

            testJobGroup = JobGroup(subscription=testSubscription)
            testJobGroup.create()

            # Create jobs
            self.makeNJobs(name=name, task=task,
                           nJobs=nJobs,
                           jobGroup=testJobGroup,
                           fileset=testFileset,
                           sub=testSubscription.exists(),
                           site=site, bl=bl, wl=wl)

            testFileset.commit()
            testJobGroup.commit()
            jobGroupList.append(testJobGroup)

        return jobGroupList

    def makeNJobs(self, name, task, nJobs, jobGroup, fileset, sub, site=None, bl=[], wl=[]):
        """
        _makeNJobs_

        Make and return a WMBS Job and File
        This handles all those damn add-ons

        """
        # Set the CacheDir
        cacheDir = os.path.join(self.testDir, 'CacheDir')

        for n in range(nJobs):
            # First make a file
            # site = self.sites[0]
            testFile = File(lfn="/singleLfn/%s/%s" % (name, n),
                            size=1024, events=10)
            if site:
                testFile.setLocation(site)
            else:
                for tmpSite in self.sites:
                    testFile.setLocation('se.%s' % (tmpSite))
            testFile.create()
            fileset.addFile(testFile)

        fileset.commit()

        index = 0
        for f in fileset.files:
            index += 1
            testJob = Job(name='%s-%i' % (name, index))
            testJob.addFile(f)
            testJob["location"] = f.getLocations()[0]
            testJob['custom']['location'] = f.getLocations()[0]
            testJob['task'] = task.getPathName()
            testJob['sandbox'] = task.data.input.sandbox
            testJob['spec'] = os.path.join(self.testDir, 'basicWorkload.pcl')
            testJob['mask']['FirstEvent'] = 101
            testJob['owner'] = 'tapas'
            testJob["siteBlacklist"] = bl
            testJob["siteWhitelist"] = wl
            testJob['ownerDN'] = 'tapas'
            testJob['ownerRole'] = 'cmsrole'
            testJob['ownerGroup'] = 'phgroup'

            jobCache = os.path.join(cacheDir, 'Sub_%i' % (sub), 'Job_%i' % (index))
            os.makedirs(jobCache)
            testJob.create(jobGroup)
            testJob['cache_dir'] = jobCache
            testJob.save()
            jobGroup.add(testJob)
            with open(os.path.join(jobCache, 'job.pkl'), 'wb') as output:
                pickle.dump(testJob, output)

        return testJob, testFile

    def createDummyJobs(self, nJobs, location=None):
        """
        _createDummyJobs_

        Create some dummy jobs
        """

        if not location:
            location = self.sites[0]

        nameStr = makeUUID()

        testWorkflow = Workflow(spec=nameStr, owner="tapas",
                                name=nameStr, task="basicWorkload/Production",
                                owner_vogroup='phgroup', owner_vorole='cmsrole')
        testWorkflow.create()

        testFileset = Fileset(name=nameStr)
        testFileset.create()

        testSubscription = Subscription(fileset=testFileset,
                                        workflow=testWorkflow,
                                        type="Processing",
                                        split_algo="FileBased")
        testSubscription.create()

        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        jobList = []

        for i in range(nJobs):
            testJob = Job(name='%s-%i' % (nameStr, i))
            testJob['location'] = location
            testJob['custom']['location'] = location
            testJob['userdn'] = 'tapas'
            testJob['owner'] = 'tapas'
            testJob['userrole'] = 'cmsrole'
            testJob['usergroup'] = 'phgroup'

            testJob.create(testJobGroup)
            jobList.append(testJob)

        return jobList

    @attr('integration')
    def testA_APITest(self):
        """
        _APITest_

        This is a commissioning test that has very little to do
        with anything except loading the code.
        """
        myThread = threading.currentThread()

        config = self.getConfig()

        baAPI = BossAirAPI(config=config, insertStates=True)

        # We should have loaded a plugin
        self.assertTrue('TestPlugin' in baAPI.plugins.keys())

        result = myThread.dbi.processData("SELECT name FROM bl_status")[0].fetchall()
        statusList = []
        for i in result:
            statusList.append(i.values()[0])

        # We should have the plugin states in the database
        self.assertEqual(statusList.sort(), ['New', 'Dead', 'Gone'].sort())

        # Create some jobs
        nJobs = 10

        jobDummies = self.createDummyJobs(nJobs=nJobs)
        print(jobDummies)

        baAPI.createNewJobs(wmbsJobs=jobDummies)

        runningJobs = baAPI._listRunJobs()

        self.assertEqual(len(runningJobs), nJobs)

        newJobs = baAPI._loadByStatus(status='New')
        self.assertEqual(len(newJobs), nJobs)
        deadJobs = baAPI._loadByStatus(status='Dead')
        self.assertEqual(len(deadJobs), 0)
        raisesException = False

        self.assertRaises(BossAirException,
                          baAPI._loadByStatus, status='FalseStatus')

        # Change the job status and update it
        for job in newJobs:
            job['status'] = 'Dead'

        baAPI._updateJobs(jobs=newJobs)

        # Test whether we see the job status as updated
        newJobs = baAPI._loadByStatus(status='New')
        self.assertEqual(len(newJobs), 0)
        deadJobs = baAPI._loadByStatus(status='Dead')
        self.assertEqual(len(deadJobs), nJobs)

        # Can we load by BossAir ID?
        loadedJobs = baAPI._loadByID(jobs=deadJobs)
        self.assertEqual(len(loadedJobs), nJobs)

        # Can we load via WMBS?
        loadedJobs = baAPI.loadByWMBS(wmbsJobs=jobDummies)
        self.assertEqual(len(loadedJobs), nJobs)

        # See if we can delete jobs
        baAPI._deleteJobs(jobs=deadJobs)

        # Confirm that they're gone
        deadJobs = baAPI._loadByStatus(status='Dead')
        self.assertEqual(len(deadJobs), 0)

        self.assertEqual(len(baAPI.jobs), 0)

        return

    @attr('integration')
    def testB_PluginTest(self):
        """
        _PluginTest_


        Now check that these functions worked if called through plugins
        Instead of directly.

        There are only three plugin
        """
        myThread = threading.currentThread()

        config = self.getConfig()

        baAPI = BossAirAPI(config=config, insertStates=True)

        # Create some jobs
        nJobs = 10

        jobDummies = self.createDummyJobs(nJobs=nJobs, location='T3_US_Xanadu')
        changeState = ChangeState(config)
        changeState.propagate(jobDummies, 'created', 'new')
        changeState.propagate(jobDummies, 'executing', 'created')

        # Prior to building the job, each job must have a plugin
        # and user assigned
        for job in jobDummies:
            job['plugin'] = 'TestPlugin'
            job['owner'] = 'tapas'

        baAPI.submit(jobs=jobDummies)

        newJobs = baAPI._loadByStatus(status='New')
        self.assertEqual(len(newJobs), nJobs)

        # Should be no more running jobs
        runningJobs = baAPI._listRunJobs()
        self.assertEqual(len(runningJobs), nJobs)

        # Test Plugin should complete all jobs
        baAPI.track()

        # Should be no more running jobs
        runningJobs = baAPI._listRunJobs()
        self.assertEqual(len(runningJobs), 0)

        # Check if they're complete
        completeJobs = baAPI.getComplete()
        self.assertEqual(len(completeJobs), nJobs)

        # Do this test because BossAir is specifically built
        # to keep it from finding completed jobs
        result = myThread.dbi.processData("SELECT id FROM bl_runjob")[0].fetchall()
        self.assertEqual(len(result), nJobs)

        jobsToRemove = baAPI._buildRunningJobs(wmbsJobs=jobDummies)
        baAPI._deleteJobs(jobs=jobsToRemove)

        result = myThread.dbi.processData("SELECT id FROM bl_runjob")[0].fetchall()
        self.assertEqual(len(result), 0)

        return

    def testG_monitoringDAO(self):
        """
        _monitoringDAO_

        Because I need a test for the monitoring DAO
        """
        myThread = threading.currentThread()

        config = self.getConfig()

        baAPI = BossAirAPI(config=config, insertStates=True)

        # Create some jobs
        nJobs = 10

        jobDummies = self.createDummyJobs(nJobs=nJobs)

        # Prior to building the job, each job must have a plugin
        # and user assigned
        for job in jobDummies:
            job['plugin'] = 'TestPlugin'
            job['owner'] = 'tapas'
            job['location'] = 'T2_US_UCSD'
            job.save()

        baAPI.submit(jobs=jobDummies)

        results = baAPI.monitor()
        self.assertEqual(results[0]['Pending'], nJobs)

        return


if __name__ == '__main__':
    unittest.main()



from __future__ import division, print_function

from builtins import bytes, str
from future.utils import viewitems

import time
from WMCore.REST.HeartbeatMonitorBase import HeartbeatMonitorBase
from WMCore.ReqMgr.DataStructs.RequestStatus import ACTIVE_STATUS
from WMCore.Services.WMStatsServer.WMStatsServer import WMStatsServer


def _getCampaign(propertyValue):
    """
    WMStats server can return this argument in three different data types.
    If there are multiple values, return only the first one.
    """
    if not propertyValue:  # Empty string is Ok; it should never be None though
        return ["NONE"]
    elif isinstance(propertyValue, (str, bytes)):
        return [propertyValue]
    # then it's a list
    return propertyValue


def _getRequestNumEvents(propertyValue):
    """
    WMStats server can return this argument in three different data types.
    If there are multiple values, return only the first one.
    """
    if not propertyValue:
        return 0
    elif isinstance(propertyValue, list):
        return propertyValue[0]
    # then it's integer
    return propertyValue


def initMetrics():
    """
    Initialize the metrics dictionary
    :param veryActiveStatus: a small list of active status
    """
    results = {"requestsByStatus": {},
               "requestsByStatusAndCampaign": {},
               "requestsByStatusAndPrio": {},
               "requestsByStatusAndNumEvts": {}}

    for st in ACTIVE_STATUS:
        results["requestsByStatus"].setdefault(st, 0)
        results["requestsByStatusAndNumEvts"].setdefault(st, 0)
        results["requestsByStatusAndCampaign"].setdefault(st, {})
        results["requestsByStatusAndPrio"].setdefault(st, {})

    return results


class HeartbeatMonitor(HeartbeatMonitorBase):

    def __init__(self, rest, config):
        super(HeartbeatMonitor, self).__init__(rest, config)
        self.producer = "reqmgr2"
        self.docTypeAMQ = "cms_%s_info" % self.producer

    def addAdditionalMonitorReport(self, config):
        """
        _addAdditionalMonitorReport_

        Collect general request information and post it to both central
        couchdb and MonIT. Items to fetch:
         * number of requests in each status (excluding archived ones)
         * for requests in assignment-approved:
           * their priority
           * their RequestNumEvents (if there is no input dataset)
         * for assignment-approved, assigned, acquired, running-* and completed:
           * number of requests in each campaign
        """
        self.logger.info("Collecting ReqMgr2 statistics...")
        wmstatsSvc = WMStatsServer(config.wmstatsSvc_url, logger=self.logger)

        results = initMetrics()

        inputConditon = {}
        outputMask = ["RequestStatus", "RequestType", "RequestPriority",
                      "Campaign", "RequestNumEvents"]
        startT = int(time.time())
        for reqInfo in wmstatsSvc.getFilteredActiveData(inputConditon, outputMask):
            status = reqInfo['RequestStatus']
            results['requestsByStatus'][status] += 1

            for campaign in _getCampaign(reqInfo["Campaign"]):
                results["requestsByStatusAndCampaign"][status].setdefault(campaign, 0)
                results['requestsByStatusAndCampaign'][status][campaign] += 1

            requestPrio = reqInfo['RequestPriority']
            results["requestsByStatusAndPrio"][status].setdefault(requestPrio, 0)
            results['requestsByStatusAndPrio'][status][requestPrio] += 1

            results['requestsByStatusAndNumEvts'][status] += _getRequestNumEvents(reqInfo['RequestNumEvents'])

        endT = int(time.time())
        results["total_query_time"] = endT - startT

        if self.postToAMQ:
            allDocs = self.buildMonITDocs(results)
            self.uploadToAMQ(allDocs)

        return results

    def buildMonITDocs(self, stats):
        """
        Given the statistics that are uploaded to wmstats, create different
        documents to post to MonIT AMQ (aggregation-friendly docs).
        """
        commonInfo = {"agent_url": "reqmgr2"}

        docs = []
        for status, numReq in viewitems(stats['requestsByStatus']):
            doc = {}
            doc["type"] = "reqmgr2_status"
            doc["request_status"] = status
            doc["num_requests"] = numReq
            doc.update((commonInfo))
            docs.append(doc)

        for status, items in viewitems(stats['requestsByStatusAndCampaign']):
            for campaign, numReq in viewitems(items):
                doc = {}
                doc["type"] = "reqmgr2_campaign"
                doc["request_status"] = status
                doc["campaign"] = campaign
                doc["num_requests"] = numReq
                doc.update((commonInfo))
                docs.append(doc)

        for status, items in viewitems(stats['requestsByStatusAndPrio']):
            for prio, numReq in viewitems(items):
                doc = {}
                doc["type"] = "reqmgr2_prio"
                doc["request_status"] = status
                doc["request_priority"] = prio
                doc["num_requests"] = numReq
                doc.update((commonInfo))
                docs.append(doc)

        for status, evts in viewitems(stats['requestsByStatusAndNumEvts']):
            doc = {}
            doc["type"] = "reqmgr2_events"
            doc["request_status"] = status
            doc["total_num_events"] = evts
            doc.update((commonInfo))
            docs.append(doc)

        self.logger.info("%i docs created to post to MonIT", len(docs))
        return docs



#!/usr/bin/env python
"""
Unittests for MathUtils functions
"""

from __future__ import division, print_function

import unittest

from Utils.MathUtils import quantize


class MathUtilsTest(unittest.TestCase):
    """
    unittest for MathUtils functions
    """

    def testQuantize(self):
        """
        Test the quantize function
        """
        self.assertEqual(quantize(15, 5), 15)
        self.assertEqual(quantize(14, 5), 15)
        self.assertEqual(quantize(16, 5), 20)

        self.assertEqual(quantize(15, 5.0), 15)
        self.assertEqual(quantize(14, 5.0), 15)
        self.assertEqual(quantize(16, 5.0), 20)

        self.assertRaises(ValueError, quantize, [1], 50)
        self.assertRaises(ValueError, quantize, {1}, 50)

        self.assertRaises(ValueError, quantize, 1, [50])
        self.assertRaises(ValueError, quantize, 1, {50})


if __name__ == "__main__":
    unittest.main()


#!/usr/bin/env python
"""
_JobTestBase

Base class for tests of WMBS job class.
"""
from __future__ import absolute_import, division, print_function

import threading
import unittest

from WMCore.DAOFactory import DAOFactory
from WMCore.DataStructs.Run import Run
from WMCore.Services.UUIDLib import makeUUID
from WMCore.WMBS.File import File
from WMCore.WMBS.Fileset import Fileset as Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMQuality.TestInit import TestInit


class JobTestBase(unittest.TestCase):
    """
    Base class for tests of WMBS job class (Job_t and JobWorkUnit_t)
    """

    def __init__(self, *args, **kwargs):
        """
        Define some class instance variables we'll fill later
        """
        super(JobTestBase, self).__init__(*args, **kwargs)

        self.testFileA = None
        self.testFileB = None
        self.testWorkflow = None
        self.testJob = None

        return

    def setUp(self):
        """
        _setUp_

        Setup the database and logging connection.  Try to create all of the
        WMBS tables.
        """

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema(customModules=["WMCore.WMBS"],
                                useDefault=False)

        myThread = threading.currentThread()
        self.daoFactory = DAOFactory(package="WMCore.WMBS",
                                     logger=myThread.logger,
                                     dbinterface=myThread.dbi)

        locationNew = self.daoFactory(classname="Locations.New")
        locationNew.execute(siteName="test.site.ch", pnn="T2_CH_CERN")
        locationNew.execute(siteName="test2.site.ch", pnn="T2_CH_CERN")

        return

    def tearDown(self):
        """
        _tearDown_

        Drop all the WMBS tables.
        """

        self.testInit.clearDatabase()

    @staticmethod
    def createTestJob(subscriptionType="Merge"):
        """
        _createTestJob_

        Create a test job with two files as input.  This will also create the
        appropriate workflow, jobgroup and subscription.
        """

        testWorkflow = Workflow(spec=makeUUID(), owner="Simon", name=makeUUID(), task="Test")
        testWorkflow.create()

        testWMBSFileset = Fileset(name="TestFileset")
        testWMBSFileset.create()

        testSubscription = Subscription(fileset=testWMBSFileset, workflow=testWorkflow, type=subscriptionType)
        testSubscription.create()

        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10)
        testFileA.addRun(Run(1, *[45]))
        testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10)
        testFileB.addRun(Run(1, *[46]))
        testFileA.create()
        testFileB.create()

        testJob = Job(name=makeUUID(), files=[testFileA, testFileB])
        testJob["couch_record"] = "somecouchrecord"
        testJob["location"] = "test.site.ch"
        testJob.create(group=testJobGroup)
        testJob.associateFiles()

        return testJob

    def createSingleJobWorkflow(self):
        """
        Create a workflow with one jobs and two files and store the results in instance variables
        """

        self.testWorkflow = Workflow(spec="spec.xml", owner="Simon", name="wf001", task="Test")
        self.testWorkflow.create()

        testWMBSFileset = Fileset(name="TestFileset")
        testWMBSFileset.create()

        testSubscription = Subscription(fileset=testWMBSFileset, workflow=self.testWorkflow)
        testSubscription.create()

        testJobGroup = JobGroup(subscription=testSubscription)
        testJobGroup.create()

        self.testFileA = File(lfn="/this/is/a/lfnA", size=1024, events=10)
        self.testFileA.addRun(Run(1, *[45]))
        self.testFileB = File(lfn="/this/is/a/lfnB", size=1024, events=10)
        self.testFileB.addRun(Run(1, *[46]))
        self.testFileA.create()
        self.testFileB.create()

        self.testJob = Job(name="TestJob", files=[self.testFileA, self.testFileB])

        self.testJob.create(group=testJobGroup)
        self.testJob.associateFiles()



#!/usr/bin/env python
"""
_Subscription_t_

Testcase for the Subscription class
"""

from builtins import str, range

import random
import unittest

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Fileset import Fileset
from WMCore.DataStructs.Run import Run
from WMCore.DataStructs.Subscription import Subscription
from WMCore.DataStructs.Workflow import Workflow


class SubscriptionTest(unittest.TestCase):
    """
    _SubscriptionTest_

    Testcase for the Subscription class

    """

    def setUp(self):
        """
        Initial Setup for Subscription Testcase

        Set a dummy Subscription with a fileset composed of one file inside it
        and a dummy workflow using the default constructor of the Workflow class

        """
        self.dummyFile = File('/tmp/dummyfile', 9999, 0, 0, 0)
        self.dummySet = set()
        self.dummySet.add(self.dummyFile)
        self.dummyFileSet = Fileset(name='SubscriptionTestFileset',
                                    files=self.dummySet)
        self.dummyWorkFlow = Workflow()
        self.dummySubscription = Subscription(fileset=self.dummyFileSet,
                                              workflow=self.dummyWorkFlow)
        return

    def tearDown(self):
        pass

    def testGetWorkflow(self):
        """
        Testcase for the getWorkflow method of the Subscription Class

        """
        assert self.dummySubscription['workflow'] == self.dummyWorkFlow, \
            'Couldn\'t add Workflow to Subscription'

    def testGetFileset(self):
        """
        Testcase for the getFileset method of the Subscription Class

        """
        assert self.dummyFileSet.name == self.dummySubscription['fileset'].name, \
            'Couldn\'t add Fileset to Subscription - name does not match'

        for x in self.dummyFileSet.listNewFiles():
            assert x in self.dummySubscription['fileset'].newfiles, \
                'Couldn\'t add Fileset to Subscription - newFiles Set does not match'

        assert self.dummyFileSet.getFiles(type='set') == \
               self.dummySubscription['fileset'].getFiles(type='set'), \
            'Couldn\'t add Fileset to Subscription - %s Set does not match' % x

    def testAcquireFiles(self):
        """
        Testcase for the acquireFiles method of the Subscription Class

        """
        # Cleaning possible files already occupying the available set
        self.dummySubscription.acquireFiles()

        # First test - Test if initial file (on available set) is inserted in the
        # acquired set - no arguments

        dummyFile2 = File('/tmp/dummyfile2,8888', 1, 1, 1)
        # Insert dummyFile2 into the available files Set at dummySubscription
        self.dummySubscription.available.addFile(dummyFile2)

        S = self.dummySubscription.available.listNewFiles()
        # Check if Set returned by method is the same that was at the previous
        # available FileSet
        assert S == self.dummySubscription.acquireFiles(), \
            'Couldn\'t acquire file using method acquireFiles - (no arguments test)'

        # Second test - Test if target files are inserted at the acquired set

        dummyFileList = set()
        # Populating the dummy List with a random number of files
        for i in range(1, random.randint(100, 1000)):
            lfn = '/store/data/%s/%s/file.root' % (random.randint(1000, 9999),
                                                   random.randint(1000, 9999))
            size = random.randint(1000, 2000)
            events = 1000
            run = random.randint(0, 2000)
            lumi = random.randint(0, 8)

            file = File(lfn=lfn, size=size, events=events,
                        checksums={"cksum": "1"})
            file.addRun(Run(run, *[lumi]))
            dummyFileList.add(file)

        # Check if return value is correct - with parameters
        acqFiles = self.dummySubscription.acquireFiles(files=dummyFileList)
        assert acqFiles == dummyFileList, \
            'Return value for acquireFiles method not the acquired files'
        # Check if all files were inserted at subscription acquired files Set
        for x in dummyFileList:
            assert x in self.dummySubscription.acquired.getFiles(type='set'), \
                'Couldn\'t acquire File %s' % x.dict['lfn']

        # Third test - Test if a replicate file is erased from the other Sets,
        # when a file is acquired

        dummyFile3 = File('/tmp/dummyfile3,5555', 1, 1, 1)
        dummyFileList = []
        dummyFileList.append(dummyFile3)

        # Inserting dummyFile3 to be used as an argument, into each of the other file sets
        self.dummySubscription.available.addFile(dummyFile3)
        self.dummySubscription.failed.addFile(dummyFile3)
        self.dummySubscription.completed.addFile(dummyFile3)

        # Run the method acquireFiles
        self.dummySubscription.acquireFiles(files=dummyFileList, size=1)

        # Check if dummyFile3 was inserted at the acquired Set
        assert dummyFile3 in self.dummySubscription.acquired.getFiles(type='set'), \
            'Replicated file could\'nt be inserted at acquired Set'

        # Check if dummyFile3 was erased from all the other Sets
        assert dummyFile3 not in self.dummySubscription.available.getFiles(type='set'), \
            'Acquired file still present at available Set'
        assert dummyFile3 not in self.dummySubscription.failed.getFiles(type='set'), \
            'Acquired file still present at failed Set'
        assert dummyFile3 not in self.dummySubscription.completed.getFiles(type='set'), \
            'Acquired file still present at completed Set'

        # Fourth test - Test if the method works properly if a wrong size number
        # is given as an argument

        # Case 1: size < number of files given as an argument

        dummyFileList = []
        for i in range(90, 100):
            dummyFileSize = File('/tmp/dummyfile' + str(i), 7656, 1, 1, 1)
            dummyFileList.append(dummyFileSize)

        # Run the method:
        self.dummySubscription.acquireFiles(files=dummyFileList, size=1)
        # Check each file of the List
        for x in dummyFileList:
            assert x in self.dummySubscription.acquired.getFiles(type='set'), \
                'File wasn\'t acquired (lower Size argument test)'

        # Case 2: size = 0

        # Run the method:
        self.dummySubscription.acquireFiles(files=dummyFileList, size=0)
        # Check each file of the List
        for x in dummyFileList:
            assert x in self.dummySubscription.acquired.getFiles(type='set'), \
                'File wasn\'t acquired (zero size argument test)'

    def testCompleteFiles(self):
        """
        Testcase for the completeFiles method of the Subscription Class

        """
        # Cleaning possible files already occupying the available set
        self.dummySubscription.completeFiles([])

        # First test - Test if initial file (on available set) is inserted in the
        # completed set - no arguments

        dummyFile2 = File('/tmp/dummyfile2,8888', 1, 1, 1)
        # Insert dummyFile2 into the available files Set at dummySubscription
        self.dummySubscription.available.addFile(dummyFile2)

        S = self.dummySubscription.availableFiles()
        # complete all files
        self.dummySubscription.completeFiles(S)

        assert len(self.dummySubscription.availableFiles()) == 0, \
            "completed subscription still has %s files, what's up with that?" % \
            len(self.dummySubscription.availableFiles())

        # Second test - Test if target files are inserted at the completed files set

        dummyFileList = []
        # Populating the dummy List with a random number of files
        for i in range(1, random.randint(100, 1000)):
            lfn = '/store/data/%s/%s/file.root' % (random.randint(1000, 9999),
                                                   random.randint(1000, 9999))
            size = random.randint(1000, 2000)
            events = 1000
            run = random.randint(0, 2000)
            lumi = random.randint(0, 8)

            file = File(lfn=lfn, size=size, events=events,
                        checksums={"cksum": "1"})
            file.addRun(Run(run, *[lumi]))
            dummyFileList.append(file)
        # Add the new files
        self.dummySubscription.available.addFile(dummyFileList)
        # and complete them
        self.dummySubscription.completeFiles(files=dummyFileList)
        # Check if return value is correct - with parameters
        assert len(self.dummySubscription.availableFiles()) == 0, \
            "completed subscription still has %s files, what's up with that?" % \
            len(self.dummySubscription.availableFiles())

        # Check if all files were inserted at subscription's completed files Set
        for x in dummyFileList:
            assert x in self.dummySubscription.completed.getFiles(type='set'), \
                'Couldn\'t make file completed %s' % x.dict['lfn']

        # Third test - Test if a replicate file is erased from the other Sets,
        # when a file is made completed

        dummyFile3 = File('/tmp/dummyfile3,5555', 1, 1, 1)
        dummyFileList = []
        dummyFileList.append(dummyFile3)

        # Inserting dummyFile3 to be used as an argument, into each of the other
        # file sets
        self.dummySubscription.acquired.addFile(dummyFile3)
        self.dummySubscription.failed.addFile(dummyFile3)
        self.dummySubscription.completed.addFile(dummyFile3)

        # Run the method completeFiles
        self.dummySubscription.completeFiles(files=dummyFileList)

        # Check if dummyFile3 was inserted at the completed Set
        assert dummyFile3 in self.dummySubscription.completed.getFiles(type='set'), \
            'Replicated file could\'nt be inserted at completed Set'

        # Check if dummyFile3 was erased from all the other Sets
        assert dummyFile3 not in self.dummySubscription.acquired.getFiles(type='set'), \
            'Completed file still present at acquired Set'
        assert dummyFile3 not in self.dummySubscription.failed.getFiles(type='set'), \
            'Completed file still present at failed Set'
        assert dummyFile3 not in self.dummySubscription.available.getFiles(type='set'), \
            'Completed file still present at available Set'

    def testFailFiles(self):
        """
        Testcase for the failFiles method of the Subscription Class

        """
        # Cleaning possible files already occupying the available set
        self.dummySubscription.failFiles([])

        # First test - Test if initial file (on available set) is inserted in the
        # failed set - no arguments

        dummyFile2 = File('/tmp/dummyfile2,8888', 1, 1, 1)
        # Insert dummyFile2 into the available files Set at dummySubscription
        self.dummySubscription.available.addFile(dummyFile2)

        S = self.dummySubscription.availableFiles()
        # Fail all files
        self.dummySubscription.failFiles(S)

        assert len(self.dummySubscription.availableFiles()) == 0, \
            "failed subscription still has %s files, what's up with that?" % \
            len(self.dummySubscription.availableFiles())

        # Second test - Test if target files are inserted at the failed set

        dummyFileList = []
        # Populating the dummy List with a random number of files
        for i in range(1, random.randint(100, 1000)):
            lfn = '/store/data/%s/%s/file.root' % (random.randint(1000, 9999),
                                                   random.randint(1000, 9999))
            size = random.randint(1000, 2000)
            events = 1000
            run = random.randint(0, 2000)
            lumi = random.randint(0, 8)

            file = File(lfn=lfn, size=size, events=events,
                        checksums={"cksum": "1"})
            file.addRun(Run(run, *[lumi]))
            dummyFileList.append(file)
        # Add the new files
        self.dummySubscription.available.addFile(dummyFileList)
        # and fail them
        self.dummySubscription.failFiles(files=dummyFileList)
        # Check there are no files available - everything should be failed
        assert len(self.dummySubscription.availableFiles()) == 0, \
            "failed subscription still has %s files, what's up with that?" % \
            len(self.dummySubscription.availableFiles())

        # Check if all files were inserted at subscription's failed files Set
        for x in dummyFileList:
            assert x in self.dummySubscription.failed.getFiles(type='set'), \
                'Couldn\'t make file failed %s' % x.dict['lfn']

        # Third test - Test if a replicate file is erased from the other Sets,
        # when a file is considered failed

        dummyFile3 = File('/tmp/dummyfile3,5555', 1, 1, 1)
        dummyFileList = []
        dummyFileList.append(dummyFile3)

        # Inserting dummyFile3 to be used as an argument, into each of the other
        # file sets
        self.dummySubscription.acquired.addFile(dummyFile3)
        self.dummySubscription.available.addFile(dummyFile3)
        self.dummySubscription.completed.addFile(dummyFile3)

        # Run the method failFiles
        self.dummySubscription.failFiles(files=dummyFileList)

        # Check if dummyFile3 was inserted at the failed Set
        assert dummyFile3 in self.dummySubscription.failed.getFiles(type='set'), \
            'Replicated file could\'nt be inserted at failed Set'

        # Check if dummyFile3 was erased from all the other Sets
        assert dummyFile3 not in self.dummySubscription.acquired.getFiles(type='set'), \
            'Failed file still present at acquired Set'
        assert dummyFile3 not in self.dummySubscription.completed.getFiles(type='set'), \
            'Failed file still present at completed Set'
        assert dummyFile3 not in self.dummySubscription.available.getFiles(type='set'), \
            'Failed file still present at available Set'

    def testFilesOfStatus(self):
        """
        Testcase for the filesOfStatus method of the Subscription Class

        """

        assert self.dummySubscription.filesOfStatus('Available') == \
               self.dummySubscription.available.getFiles(type='set') - \
               self.dummySubscription.acquiredFiles() | self.dummySubscription.completedFiles() | self.dummySubscription.failedFiles(), \
            'Method fileOfStatus(\'AvailableFiles\') does not return available files set'
        assert self.dummySubscription.filesOfStatus('Acquired') == self.dummySubscription.acquired.getFiles(type='set'), \
            'Method fileOfStatus(\'AcquiredFiles\') does not return acquired files set'
        assert self.dummySubscription.filesOfStatus('Completed') == self.dummySubscription.completed.getFiles(
            type='set'), \
            'Method fileOfStatus(\'CompletedFiles\') does not return completed files set'
        assert self.dummySubscription.filesOfStatus('Failed') == self.dummySubscription.failed.getFiles(type='set'), \
            'Method fileOfStatus(\'FailedFiles\') does not return failed files set'

    def testAvailableFiles(self):
        """
        Testcase for the availableFiles method of the Subscription Class
        """
        assert self.dummySubscription.availableFiles() == \
               self.dummySubscription.available.getFiles(type='set'), \
            'Method availableFiles does not return available files Set'

    def testAcquiredFiles(self):
        """
        Testcase for the acquiredFiles method of the Subscription Class
        """
        assert self.dummySubscription.acquiredFiles() == \
               self.dummySubscription.acquired.getFiles(type='set'), \
            'Method acquiredFiles does not return acquired files Set'

    def testCompletedFiles(self):
        """
        Testcase for the completedFiles method of the Subscription Class
        """
        assert self.dummySubscription.completedFiles() == \
               self.dummySubscription.completed.getFiles(type='set'), \
            'Method completedFiles does not return completed files Set'

    def testFailedFiles(self):
        """
        Testcase for the failedFiles method of the Subscription Class
        """
        assert self.dummySubscription.failedFiles() == \
               self.dummySubscription.failed.getFiles(type='set'), \
            'Method failedFiles does not return failed files Set'


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
Test case for UserFileCache
"""

import unittest
import filecmp
import os
from os import path

from WMCore.Services.UserFileCache.UserFileCache import UserFileCache, calculateChecksum
from WMCore.WMBase import getTestBase

class UserFileCacheTest(unittest.TestCase):
    """
    Unit tests for UserFileCache Service
    """


    def testChecksum(self):
        """
        Tests checksum method
        """
        checksum1 = calculateChecksum(tarfile_=path.join(getTestBase(), 'WMCore_t/Services_t/UserFileCache_t/ewv_crab_EwvAnalysis_31_111229_140959_publish.tgz'))
        checksum2 = calculateChecksum(tarfile_=path.join(getTestBase(), 'WMCore_t/Services_t/UserFileCache_t/ewv_crab_EwvAnalysis_31_resubmit_111229_144319_publish.tgz'))
        self.assertTrue(checksum1)
        self.assertTrue(checksum2)
        self.assertFalse(checksum1 == checksum2)

        self.assertRaises(IOError, calculateChecksum, **{'tarfile_': 'does_not_exist'})
        return


    def testUploadDownload(self):
        if 'UFCURL' in os.environ:
            currdir = getTestBase()
            upfile = path.join(currdir, 'WMCore_t/Services_t/UserFileCache_t/test_file.tgz') #file to upload
            upfileLog = path.join(currdir, 'WMCore_t/Services_t/UserFileCache_t/uplog.txt') #file to upload
            ufc = UserFileCache({'endpoint':os.environ['UFCURL'], 'pycurl': True})

            #hashkey upload/download
            res = ufc.upload(upfile)
            ufc.download(res['hashkey'], output='pippo_publish_down.tgz')

            #hashkey deletion
            ufc.removeFile(res['hashkey'])

            #log upload/download
            res = ufc.uploadLog(upfileLog)
            ufc.downloadLog(upfileLog, upfileLog+'.downloaded')
            self.assertTrue(filecmp.cmp(upfileLog, upfileLog+'.downloaded'))



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
"""
File       : CPMetrics.py
Author     : Valentin Kuznetsov <vkuznet AT gmail dot com>
Description: This module provide functions to flatten cherrypy stats
and provide them in Prometheus format for scraping.

Below we present three different outputs:
    - CherryPy stats default output
    - flatten structure of stats
    - prometheus output for stats

### Defautl CherryPy metrics output

{
    "Cheroot HTTPServer 4388603856": {
    "Accepts": 0,
    "Accepts/sec": 0.0,
    "Bind Address": "('127.0.0.1', 8080)",
    "Bytes Read": -1,
    "Bytes Written": -1,
    "Enabled": false,
    "Queue": 0,
    "Read Throughput": -1,
    "Requests": -1,
    "Run time": -1,
    "Socket Errors": 0,
    "Threads": 20,
    "Threads Idle": 19,
    "Work Time": -1,
    "Worker Threads": {
        "CP Server Thread-10": {
        "Bytes Read": 0,
        "Bytes Written": 0,
        "Read Throughput": 0.0,
        "Requests": 0,
        "Work Time": 0,
        "Write Throughput": 0.0
        },
        .....
    },
    "Write Throughput": -1
    },
    "CherryPy Applications": {
        "Bytes Read/Request": 0.0,
        "Bytes Read/Second": 0.0,
        "Bytes Written/Request": 0.0,
        "Bytes Written/Second": 0.0,
        "Current Requests": 0,
        "Current Time": 1601039589.916887,
        "Enabled": true,
        "Requests": {},
        "Requests/Second": 0.0,
        "Server Version": "17.4.2",
        "Start Time": 1601039576.541158,
        "Total Bytes Read": 0,
        "Total Bytes Written": 0,
        "Total Requests": 0,
        "Total Time": 0,
        "Uptime": 13.375718116760254
    }
}

### flatten structure via flattenStats function
{
"cherrypy_app_requests_second": 0.0,
"cherrypy_app_uptime": 1.7454230785369873,
"cherrypy_app_total_time": 0,
"cherrypy_app_current_time": 1601040713.369842,
"cherrypy_app_bytes_read_second": 0.0,
"cherrypy_app_requests": {},
"cherrypy_http_server_queue": 0,
"cherrypy_app_bytes_written_second": 0.0,
"cherrypy_app_total_requests": 0,
"cherrypy_http_server_write_throughput": -1,
"cherrypy_app_enabled": true,
"cherrypy_app_start_time": 1601040711.624408,
"cherrypy_http_server_threads": 20,
"cherrypy_http_server_work_time": -1,
"cherrypy_app_bytes_read_request": 0.0,
"cherrypy_http_server_bytes_read": -1,
"cherrypy_http_server_accepts_sec": 0.0,
"cherrypy_server_worker_threads": [
   {"thread_name": "cp_server_thread_3", "read_throughput": 0.0,
   "work_time": 0, "write_throughput": 0.0, "bytes_written": 0,
   "bytes_read": 0, "requests": 0}, ....],
"cherrypy_http_server_run_time": -1,
"cherrypy_http_server_bind_address": "('127.0.0.1', 8080)",
"cherrypy_app_server_version": "17.4.2",
"cherrypy_http_server_enabled": false,
"cherrypy_http_server_socket_errors": 0,
"cherrypy_app_bytes_written_request": 0.0,
"cherrypy_app_current_requests": 0,
"cherrypy_app_total_bytes_read": 0,
"cherrypy_http_server_bytes_written": -1,
"cherrypy_http_server_threads_idle": 19,
"cherrypy_http_server_requests": -1,
"cherrypy_app_total_bytes_written": 0,
"cherrypy_http_server_read_throughput": -1,
"cherrypy_http_server_accepts": 0}

### prometheus exporter structure provided via promMetrics function

# HELP cherrypy_app_requests_second
# TYPE cherrypy_app_requests_second gauge
cherrypy_app_requests_second 0.0
# HELP cherrypy_app_uptime
# TYPE cherrypy_app_uptime gauge
cherrypy_app_uptime 2.03766894341
# HELP cherrypy_app_total_time
# TYPE cherrypy_app_total_time counter
cherrypy_app_total_time 0
# HELP cherrypy_app_current_time
# TYPE cherrypy_app_current_time gauge
cherrypy_app_current_time 1601043839.47
# HELP cherrypy_app_bytes_read_second
# TYPE cherrypy_app_bytes_read_second gauge
cherrypy_app_bytes_read_second 0.0
# HELP cherrypy_http_server_queue
# TYPE cherrypy_http_server_queue counter
cherrypy_http_server_queue 0
# HELP cherrypy_app_bytes_written_second
# TYPE cherrypy_app_bytes_written_second gauge
cherrypy_app_bytes_written_second 0.0
# HELP cherrypy_app_total_requests
# TYPE cherrypy_app_total_requests counter
cherrypy_app_total_requests 0
# HELP cherrypy_http_server_write_throughput
# TYPE cherrypy_http_server_write_throughput counter
cherrypy_http_server_write_throughput -1
# HELP cherrypy_server_worker_threads
# TYPE cherrypy_server_worker_threads histogram
cherrypy_server_worker_threads{thread_name="cp_server_thread_3",...} 1
# HELP cherrypy_app_start_time
# TYPE cherrypy_app_start_time gauge
cherrypy_app_start_time 1601043837.44
# HELP cherrypy_http_server_threads
# TYPE cherrypy_http_server_threads counter
cherrypy_http_server_threads 20
# HELP cherrypy_http_server_work_time
# TYPE cherrypy_http_server_work_time counter
cherrypy_http_server_work_time -1
# HELP cherrypy_app_bytes_read_request
# TYPE cherrypy_app_bytes_read_request gauge
cherrypy_app_bytes_read_request 0.0
# HELP cherrypy_http_server_bytes_read
# TYPE cherrypy_http_server_bytes_read counter
cherrypy_http_server_bytes_read -1
# HELP cherrypy_http_server_accepts_sec
# TYPE cherrypy_http_server_accepts_sec gauge
cherrypy_http_server_accepts_sec 0.0
# HELP cherrypy_http_server_run_time
# TYPE cherrypy_http_server_run_time counter
cherrypy_http_server_run_time -1
# HELP cherrypy_http_server_socket_errors
# TYPE cherrypy_http_server_socket_errors counter
cherrypy_http_server_socket_errors 0
# HELP cherrypy_app_bytes_written_request
# TYPE cherrypy_app_bytes_written_request gauge
cherrypy_app_bytes_written_request 0.0
# HELP cherrypy_app_current_requests
# TYPE cherrypy_app_current_requests counter
cherrypy_app_current_requests 0
# HELP cherrypy_app_total_bytes_read
# TYPE cherrypy_app_total_bytes_read counter
cherrypy_app_total_bytes_read 0
# HELP cherrypy_http_server_bytes_written
# TYPE cherrypy_http_server_bytes_written counter
cherrypy_http_server_bytes_written -1
# HELP cherrypy_http_server_threads_idle
# TYPE cherrypy_http_server_threads_idle counter
cherrypy_http_server_threads_idle 19
# HELP cherrypy_http_server_requests
# TYPE cherrypy_http_server_requests counter
cherrypy_http_server_requests -1
# HELP cherrypy_app_total_bytes_written
# TYPE cherrypy_app_total_bytes_written counter
cherrypy_app_total_bytes_written 0
# HELP cherrypy_http_server_read_throughput
# TYPE cherrypy_http_server_read_throughput counter
cherrypy_http_server_read_throughput -1
# HELP cherrypy_http_server_accepts
# TYPE cherrypy_http_server_accepts counter
cherrypy_http_server_accepts 0
"""

# system modules
import json

# WMCore dependencies
from Utils.Utilities import decodeBytesToUnicode


def promMetrics(data, exporter):
    """
    Provide cherrypy stats prometheus metrics for given exporter name.
    """
    # exporter name should not contain dashes, see
    # https://its.cern.ch/jira/browse/CMSMONIT-514
    exporter = exporter.replace("-", "_")
    metrics = flattenStats(data)
    if isinstance(metrics, str):
        metrics = json.loads(metrics)
    # the following keys will be skipped
    skip = [
        'cherrypy_app_enabled',
        'cherrypy_http_server_bind_address',
        'cherrypy_app_requests',
        'cherrypy_app_server_version',
        'cherrypy_http_server_enabled']
    # our prometheus data representation
    pdata = ""
    for key, val in metrics.items():
        if key in skip:
            continue
        # add exporter name as a prefix for each key
        key = '{}_{}'.format(exporter, key)
        mhelp = "# HELP {}\n".format(key)
        if isinstance(val, list):
            mtype = "# TYPE {} histogram\n".format(key)
            pdata += mhelp
            pdata += mtype
            for wdict in val:
                entries = []
                for kkk, vvv in wdict.items():
                    entries.append('{}="{}"'.format(kkk, vvv))
                entry = "{%s}" % ','.join(entries)
                pdata += "{}{} 1\n".format(key, entry)
        elif isinstance(val, (str, tuple)):
            continue
        else:
            pdata += mhelp
            if isinstance(val, int):
                mtype = "# TYPE {} counter\n".format(key)
            if isinstance(val, float):
                mtype = "# TYPE {} gauge\n".format(key)
            pdata += mtype
            pdata += "{} {}\n".format(key, val)
    return pdata


def flattenStats(cpdata):
    "Flatten cherrypy stats to make them suitable for monitoring"
    if isinstance(cpdata, str) or isinstance(cpdata, bytes):
        cpdata = json.loads(decodeBytesToUnicode(cpdata))
    data = {}
    for cpKey, cpVal in cpdata.items():
        if cpKey.lower().find('cherrypy') != -1:
            for cpnKey, cpnVal in cpVal.items():
                nkey = 'cherrypy_app_%s' % cpnKey
                nkey = nkey.lower().replace(" ", "_").replace("/", "_")
                data[nkey] = cpnVal
        if cpKey.lower().find('cheroot') != -1:
            for cpnKey, cpnVal in cpVal.items():
                if cpnKey == 'Worker Threads':
                    wdata = []
                    for workerKey, threadValue in cpnVal.items():
                        workerKey = workerKey.lower().replace(" ", "_").replace("/", "_").replace("-", "_")
                        threadValue['thread_name'] = workerKey
                        nval = {}
                        for tkey, tval in threadValue.items():
                            tkey = tkey.lower().replace(" ", "_").replace("/", "_")
                            nval[tkey] = tval
                        wdata.append(nval)
                    data["cherrypy_server_worker_threads"] = wdata
                else:
                    nkey = 'cherrypy_http_server_%s' % cpnKey
                    nkey = nkey.lower().replace(" ", "_").replace("/", "_")
                    data[nkey] = cpnVal
    return data



from __future__ import division, print_function

from builtins import str
from functools import wraps
import logging
import threading

### Don't crash the components on the following database exceptions
# ORA-03113: end-of-file on communication channel
# ORA-03114: Not Connected to Oracle
# ORA-03135: connection lost contact
# ORA-12545: Connect failed because target host or object does not exist
# ORA-00060: deadlock detected while waiting for resource
# ORA-01033: ORACLE initialization or shutdown in progress
# (cx_Oracle.InterfaceError) not connected  # same as ORA-03114, in the new SQLAlchemy
# (cx_Oracle.DatabaseError) ORA-25408: can not safely replay call
# and those two MySQL exceptions
DB_CONNECTION_ERROR_STR = ["ORA-03113", "ORA-03114", "ORA-03135", "ORA-12545", "ORA-00060", "ORA-01033",
                           "MySQL server has gone away", "Lock wait timeout exceeded",
                           "(cx_Oracle.InterfaceError) not connected", "ORA-25408"]


def db_exception_handler(f):
    """
    :param f: function
    :return: wrapper fuction

    Only need to handle DB connection problem other db problems which need to rollback the transaction,
    shouldn't be included in this hanldler

    Warning: This only used, when original function return values are not used, or wrapped around Utils.Timer.timeFunc
    """

    @wraps(f)
    def wrapper(*args, **kwargs):
        myThread = threading.currentThread()
        if hasattr(myThread, "logdbClient") and myThread.logdbClient is not None:
            myThread.logdbClient.delete("DBConnection_error_handler", "warning", this_thread=True)
        try:
            return f(*args, **kwargs)
        except Exception as ex:
            msg = str(ex)
            for errStr in DB_CONNECTION_ERROR_STR:
                if errStr in msg:
                    logging.error("%s: Temp error will try later", msg)

                    if hasattr(myThread, "logdbClient") and myThread.logdbClient is not None:
                        myThread.logdbClient.post("DBConnection_error_handler", "warning")

                    # returns tuples of 3 since timeFunc returns (time to spend, result, func.__name__)
                    return 0, None, f.__name__

            # for other case raise the same exception
            raise

    return wrapper



#!/usr/bin/env python

"""
BossAir preliminary test
"""

from builtins import range
import threading
import unittest

from Utils.PythonVersion import PY3

from WMCore.BossAir.RunJob import RunJob
from WMCore.DAOFactory import DAOFactory
from WMCore.ResourceControl.ResourceControl import ResourceControl
from WMCore.WMBS.Fileset import Fileset
from WMCore.WMBS.Job import Job
from WMCore.WMBS.JobGroup import JobGroup
from WMCore.WMBS.Subscription import Subscription
from WMCore.WMBS.Workflow import Workflow
from WMQuality.TestInit import TestInit


class RunJobTest(unittest.TestCase):
    """
    _RunJobTest_


    Test the RunJob object and accessors
    """


    def setUp(self):

        myThread = threading.currentThread()

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        #self.tearDown()
        self.testInit.setSchema(customModules = ["WMCore.WMBS", "WMCore.BossAir", "WMCore.ResourceControl", "WMCore.Agent.Database"],
                                useDefault = False)

        self.daoFactory = DAOFactory(package = "WMCore.BossAir",
                                     logger = myThread.logger,
                                     dbinterface = myThread.dbi)

        resourceControl = ResourceControl()
        resourceControl.insertSite(siteName = 'Xanadu', pnn = 'se.Xanadu',
                                   ceName = 'Xanadu', plugin = "TestPlugin")
        resourceControl.insertThreshold(siteName = 'Xanadu', taskType = 'Processing', \
                                        maxSlots = 10000, pendingSlots = 10000)

        # Create user
        wmbsFactory = DAOFactory(package = "WMCore.WMBS",
                                 logger = myThread.logger,
                                 dbinterface = myThread.dbi)
        newuser = wmbsFactory(classname = "Users.New")
        newuser.execute(dn = "mnorman", group_name = "phgroup", role_name = "cmsrole")

        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def tearDown(self):
        """
        Database deletion
        """
        self.testInit.clearDatabase(modules = ["WMCore.WMBS", "WMCore.BossAir", "WMCore.ResourceControl", "WMCore.Agent.Database"])

        self.testInit.delWorkDir()

        return


    def createJobs(self, nJobs):
        """
        Creates a series of jobGroups for submissions

        """

        testWorkflow = Workflow(spec = "dummy", owner = "mnorman",
                                name = "dummy", task="basicWorkload/Production")
        testWorkflow.create()

        # Create Fileset, Subscription, jobGroup
        testFileset = Fileset(name = "dummy")
        testFileset.create()
        testSubscription = Subscription(fileset = testFileset,
                                            workflow = testWorkflow,
                                            type = "Processing",
                                            split_algo = "FileBased")
        testSubscription.create()

        testJobGroup = JobGroup(subscription = testSubscription)
        testJobGroup.create()


        # Create jobs
        for id in range(nJobs):
            testJob = Job(name = 'Job_%i' % (id))
            testJob['owner']    = "mnorman"
            testJob['location'] = 'Xanadu'
            testJob.create(testJobGroup)
            testJobGroup.add(testJob)

        testFileset.commit()
        testJobGroup.commit()


        return testJobGroup





    def testA_BulkDAOs(self):
        """
        _BulkDAOs_

        Test the bulk DAO options, which is the only thing we should be using.
        """

        myThread = threading.currentThread()

        jobGroup = self.createJobs(nJobs = 10)

        runJobs = []
        for job in jobGroup.jobs:
            runJob = RunJob(jobid = job.exists())
            runJob['status']    = 'New'
            runJob['userdn']    = job['owner']
            runJob['usergroup'] = 'phgroup'
            runJob['userrole']  = 'cmsrole'
            runJobs.append(runJob)


        statusDAO = self.daoFactory(classname = "NewState")
        statusDAO.execute(states = ['New', 'Gone', 'Dead'])

        result = myThread.dbi.processData("SELECT name FROM bl_status")[0].fetchall()
        self.assertItemsEqual(result, [('Dead',), ('Gone',), ('New',)])


        newJobDAO = self.daoFactory(classname = "NewJobs")
        newJobDAO.execute(jobs = runJobs)


        result = myThread.dbi.processData("SELECT wmbs_id FROM bl_runjob")[0].fetchall()
        self.assertEqual(result,
                         [(1,), (2,), (3,), (4,), (5,), (6,), (7,), (8,), (9,), (10,)])


        loadJobsDAO = self.daoFactory(classname = "LoadByStatus")
        loadJobs = loadJobsDAO.execute(status = "New")
        self.assertEqual(len(loadJobs), 10)


        idList = [x['id'] for x in loadJobs]

        for job in loadJobs:
            job['bulkid'] = 1001

        updateDAO = self.daoFactory(classname = "UpdateJobs")
        updateDAO.execute(jobs = loadJobs)

        loadJobs = loadJobsDAO.execute(status = 'New')
        self.assertEqual(len(loadJobs), 10)
        for job in loadJobs:
            self.assertEqual(job['bulkid'], '1001')


        loadWMBSDAO = self.daoFactory(classname = "LoadByWMBSID")
        for job in jobGroup.jobs:
            jDict = loadWMBSDAO.execute(jobs = [job])
            self.assertEqual(job['id'], jDict[0]['jobid'])


        setStatusDAO = self.daoFactory(classname = "SetStatus")
        setStatusDAO.execute(jobs = idList, status = 'Dead')

        result = loadJobsDAO.execute(status = 'Dead')
        self.assertEqual(len(result), 10)
        result = loadJobsDAO.execute(status = 'New')
        self.assertEqual(len(result), 0)

        runningJobDAO = self.daoFactory(classname = "LoadRunning")
        runningJobs = runningJobDAO.execute()

        self.assertEqual(len(runningJobs), 10)

        completeDAO = self.daoFactory(classname = "CompleteJob")
        completeDAO.execute(jobs = idList)

        result = loadJobsDAO.execute(status = 'Dead', complete = '0')
        self.assertEqual(len(result), 10)


        deleteDAO = self.daoFactory(classname = "DeleteJobs")
        deleteDAO.execute(jobs = idList)

        result = loadJobsDAO.execute(status = 'Dead')
        self.assertEqual(len(result), 0)



        return


    def testB_CheckWMBSBuild(self):
        """
        _CheckWMBSBuild_

        Trivial test that checks whether we can build
        runJobs from WMBS jobs
        """

        jobGroup = self.createJobs(nJobs = 10)

        for job in jobGroup.jobs:
            rj = RunJob()
            rj.buildFromJob(job = job)
            self.assertEqual(job['id'], rj['jobid'])
            self.assertEqual(job['retry_count'], rj['retry_count'])
            job2 = rj.buildWMBSJob()
            self.assertEqual(job['id'], job2['id'])
            self.assertEqual(job['retry_count'], job2['retry_count'])


        return

    def testC_CheckWMBSBuildRoleAndGroup(self):
        """
        _CheckWMBSBuild_

        Trivial test that checks whether we can build
        runJobs from WMBS jobs
        """
        jobGroup = []

        # Create jobs
        for id in range(10):
            testJob = Job( name = 'Job_%i' % (id) )
            testJob['owner']    = "mnorman"
            testJob['usergroup'] = "mygroup_%i" % id
            testJob['userrole'] = "myrole_%i" % id
            testJob['location'] = 'Xanadu'
            jobGroup.append(testJob)

        for job in jobGroup:
            rj = RunJob()
            rj.buildFromJob(job = job)
            self.assertEqual(job['usergroup'], rj['usergroup'])
            self.assertEqual(job['userrole'], rj['userrole'])
            job2 = rj.buildWMBSJob()
            self.assertEqual(job['usergroup'], job2['usergroup'])
            self.assertEqual(job['userrole'], job2['userrole'])
        return




if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
_GFAL2Impl_
Implementation of StageOutImpl interface for gfal-copy
"""
import argparse
import os

from WMCore.Storage.Registry import registerStageOutImpl
from WMCore.Storage.StageOutImpl import StageOutImpl

_CheckExitCodeOption = True


class GFAL2Impl(StageOutImpl):
    """
    _GFAL2Impl_
    Implement interface for GFAL2 commands (gfal-copy, gfal-rm)
    """

    def __init__(self, stagein=False):
        StageOutImpl.__init__(self, stagein)
        # If we want to execute commands in clean shell, we can`t separate them with ';'.
        # Next commands after separation are executed without env -i and this leads us with
        # mixed environment with COMP and system python.
        # GFAL2 is not build under COMP environment and it had failures with mixed environment.
        self.setups = "env -i X509_USER_PROXY=$X509_USER_PROXY JOBSTARTDIR=$JOBSTARTDIR bash -c '%s'"
        self.removeCommand = self.setups % '. $JOBSTARTDIR/startup_environment.sh; date; gfal-rm -t 600 %s '
        self.copyCommand = self.setups % '. $JOBSTARTDIR/startup_environment.sh; date; gfal-copy -t 2400 -T 2400 -p %(checksum)s %(options)s %(source)s %(destination)s'

    def createFinalPFN(self, pfn):
        """
        _createFinalPFN_
        GFAL2 requires file:/// for any direction transfers
        """
        if pfn.startswith('file:'):
            return pfn
        elif os.path.isfile(pfn):
            return "file://%s" % os.path.abspath(pfn)
        elif pfn.startswith('/'):
            return "file://%s" % os.path.abspath(pfn)
        return pfn

    def createSourceName(self, protocol, pfn):
        """
        _createSourceName_
        GFAL2 uses file:/// urls
        """
        return self.createFinalPFN(pfn)

    def createTargetName(self, protocol, pfn):
        """
        _createTargetName_
        GFAL2 uses file:// urls
        """
        return self.createFinalPFN(pfn)

    def createOutputDirectory(self, targetPFN):
        """
        _createOutputDirectory_
        gfal-copy handles this with option -p and
        it is not needed to create directory
        """
        return

    def createRemoveFileCommand(self, pfn):
        """
        _createRemoveFileCommand_
        handle file remove using gfal-rm

        gfal-rm options used:
          -t   global timeout for the execution of the command.
               Command is interrupted if time expires before it finishes
        """
        if os.path.isfile(pfn):
            return "/bin/rm -f %s" % os.path.abspath(pfn)
        else:
            return self.removeCommand % self.createFinalPFN(pfn)

    def createStageOutCommand(self, sourcePFN, targetPFN, options=None, checksums=None):
        """
        _createStageOutCommand_
        Build a gfal-copy command

        gfal-copy options used:
          -t   maximum time for the operation to terminate
          -T   global timeout for the transfer operation
          -p   if the destination directory does not exist, create it
          -K   checksum algorithm to use, or algorithm:value
        """
        result = "#!/bin/bash\n"

        copyCommandDict = {'checksum': '', 'options': '', 'source': '', 'destination': ''}

        useChecksum = (checksums is not None and 'adler32' in checksums and not self.stageIn)

        if not options:
            options = ''

        parser = argparse.ArgumentParser()
        parser.add_argument('--nochecksum', action='store_true')
        args, unknown = parser.parse_known_args(options.split())

        if not args.nochecksum:
            if useChecksum:
                checksums['adler32'] = "%08x" % int(checksums['adler32'], 16)
                copyCommandDict['checksum'] = "-K adler32:%s" % checksums['adler32']
            else:
                copyCommandDict['checksum'] = "-K adler32"

        copyCommandDict['options'] = ' '.join(unknown)

        copyCommandDict['source'] = self.createFinalPFN(sourcePFN)
        copyCommandDict['destination'] = self.createFinalPFN(targetPFN)

        copyCommand = self.copyCommand % copyCommandDict
        result += copyCommand

        if _CheckExitCodeOption:
            result += """
            EXIT_STATUS=$?
            echo "gfal-copy exit status: $EXIT_STATUS"
            if [[ $EXIT_STATUS != 0 ]]; then
               echo "ERROR: gfal-copy exited with $EXIT_STATUS"
               echo "Cleaning up failed file:"
               %s
            fi
            exit $EXIT_STATUS
            """ % self.createRemoveFileCommand(targetPFN)

        return result

    def removeFile(self, pfnToRemove):
        """
        _removeFile_
        CleanUp pfn provided
        """
        if os.path.isfile(pfnToRemove):
            command = "/bin/rm -f %s" % os.path.abspath(pfnToRemove)
        else:
            command = self.removeCommand % self.createFinalPFN(pfnToRemove)
        self.executeCommand(command)


registerStageOutImpl("gfal2", GFAL2Impl)



#!/usr/bin/env python
"""
_GetNumberOfJobsPerSite_

Oracle implementation of Jobs.GetNumberOfJobsPerSite
"""

__all__ = []



import logging

from WMCore.WMBS.MySQL.Subscriptions.GetNumberOfJobsPerSite import GetNumberOfJobsPerSite as MySQLGNOJPS

class GetNumberOfJobsPerSite(MySQLGNOJPS):
    """
    Right now the same as the MySQL implementation

    """



#!/usr/bin/env python
"""
_AcquireFiles_

Oracle implementation of Subscription.AcquireFiles
"""

from WMCore.WMBS.MySQL.Subscriptions.AcquireFiles import AcquireFiles \
     as AcquireFilesMySQL

class AcquireFiles(AcquireFilesMySQL):
    sql = """INSERT INTO wmbs_sub_files_acquired (subscription, fileid)
               SELECT :subscription, :fileid FROM dual WHERE NOT EXISTS
                 (SELECT fileid FROM wmbs_sub_files_acquired
                    WHERE fileid = :fileid AND subscription = :subscription)"""



#!/usr/bin/env python
#pylint: disable=E1101,C0103,R0902
"""
_ThreadPool_t_

Unit tests for threadpool.

"""
from __future__ import absolute_import
from __future__ import print_function




from builtins import str, range
import unittest
import threading
import time

from WMCore.ThreadPool.ThreadPool import ThreadPool

from WMQuality.TestInit import TestInit

# local import
from .Dummy import Dummy
import nose
class ThreadPoolTest(unittest.TestCase):
    """
    _ThreadPool_t_

    Unit tests for threadpool

    """
    _nrOfThreads = 10
    _nrOfPools = 5

    def setUp(self):
        "make a logger instance and create tables"

        self.testInit = TestInit(__file__)
        self.testInit.setLogging()
        self.testInit.setDatabaseConnection()
        self.testInit.setSchema()



    def tearDown(self):
        """
        Deletion of database
        """
        # FIXME: this might not work if your not using socket.

        self.testInit.clearDatabase()

    def testA(self):
        """
        __testSubscribe__

        Test subscription of a component.
        """
        raise nose.SkipTest

        myThread = threading.currentThread()
        # create a 'fake' component that contains a arg dictionary.
        component = Dummy()
        # load default parameters.
        config = self.testInit.getConfiguration()
        # normally assigned by the harness of the test component.
        config.Agent.componentName = "TestComponent"

        component.config = config

        threadPools = []
        for i in range(0, ThreadPoolTest._nrOfPools):
            threadPool = ThreadPool("WMCore.ThreadPool.ThreadSlave", \
                component, 'MyPool_'+str(i), ThreadPoolTest._nrOfThreads)
            threadPools.append(threadPool)

        # this is how you would use the threadpool. The threadpool retrieves
        # events/payloads from the message service. If a thread is available
        # it is dispatched, otherwise it is stored in the trheadpool.
        # make the number of tasks bigger than number of threads to tesT
        # the persistent queue.
        for i in range(0, ThreadPoolTest._nrOfThreads*10):
            event = 'eventNr_'+str(i)
            payload = 'payloadNr_'+str(i)
            # normally you would have different events per threadpool and
            # even different objects per pool. the payload part will be
            # pickled into the database enabling flexibility in passing
            # information.
            for j in range(0, ThreadPoolTest._nrOfPools):
                threadPools[j].enqueue(event, \
                    {'event' : event, 'payload' : payload})

        # this commit you want to be in the agent harness, so the message is
        # actual removed from the msgService. we can do this as the threadpool
        # acts as a dispatcher and is a shortlived action: dispatch to thread
        # or queue and tell agent harness it is finished.
        finished = False
        timeout = 60 # secs
        currenttime = 0
        while not finished:
            print('waiting for threads to finishs. Work left:')
            for j in range(0, ThreadPoolTest._nrOfPools):
                print('pool_'+str(j)+ ':' + str(threadPools[j].callQueue))
            time.sleep(1)
            finished = True
            currenttime += 1
            if (timeout == currenttime):
                raise RuntimeError
            for j in range(0, ThreadPoolTest._nrOfPools):
                if (len(threadPools[j].resultsQueue) < ThreadPoolTest._nrOfThreads*10):
                    finished = False
                    break
        # check if the tables are really empty and all messages
        # have been processed.
        for j in range(0, ThreadPoolTest._nrOfPools):
            assert len(threadPools[j].resultsQueue) == \
                ThreadPoolTest._nrOfThreads*10
        myThread.transaction.begin()
        for j in range(0, ThreadPoolTest._nrOfPools):
            self.assertEqual( threadPools[j].countMessages() ,  0 )
        myThread.transaction.commit()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python
"""
_GetByLFN_

MySQL implementation of DBSBufferFiles.GetByLFN
"""




from WMCore.Database.DBFormatter import DBFormatter

class GetByLFN(DBFormatter):
    sql = """SELECT files.id AS id, files.lfn AS lfn, files.filesize AS filesize,
                    files.events AS events,
                    files.status AS status,
                    dbsbuffer_algo.app_name AS app_name, dbsbuffer_algo.app_ver AS app_ver,
                    dbsbuffer_algo.app_fam AS app_fam, dbsbuffer_algo.pset_hash AS pset_hash,
                    dbsbuffer_algo.config_content, dbsbuffer_dataset.path AS dataset_path
             FROM dbsbuffer_file files
             INNER JOIN dbsbuffer_algo_dataset_assoc ON
               files.dataset_algo = dbsbuffer_algo_dataset_assoc.id
             INNER JOIN dbsbuffer_algo ON
               dbsbuffer_algo_dataset_assoc.algo_id = dbsbuffer_algo.id
             INNER JOIN dbsbuffer_dataset ON
               dbsbuffer_algo_dataset_assoc.dataset_id = dbsbuffer_dataset.id
             WHERE lfn = :lfn"""

    def format(self, result):
        """
        _format_

        Some databases (Oracle) aren't case sensitive with respect to column
        names so we'll do some formatting so the column names are returned as
        expected.
        """
        resultDict = self.formatDict(result)[0]
        resultDict["appName"] = resultDict["app_name"]
        del resultDict["app_name"]

        resultDict["appVer"] = resultDict["app_ver"]
        del resultDict["app_ver"]

        resultDict["appFam"] = resultDict["app_fam"]
        del resultDict["app_fam"]

        resultDict["psetHash"] = resultDict["pset_hash"]
        del resultDict["pset_hash"]

        resultDict["configContent"] = resultDict["config_content"]
        del resultDict["config_content"]

        resultDict["datasetPath"] = resultDict["dataset_path"]
        del resultDict["dataset_path"]

        resultDict["size"] = resultDict["filesize"]
        del resultDict["filesize"]

        return resultDict

    def getBinds(self, files):
        binds = []
        files = self.dbi.makelist(files)
        for f in files:
            binds.append({"lfn": f})
        return binds

    def execute(self, files, conn = None, transaction = False):
        binds = self.getBinds(files)
        result = self.dbi.processData(self.sql, binds, conn = conn,
                                      transaction = transaction)
        return self.format(result)



'''

'''
from __future__ import (division, print_function)

from WMCore.REST.CherryPyPeriodicTask import CherryPyPeriodicTask
from WMCore.WMStats.DataStructs.DataCache import DataCache
from WMCore.Services.WMStats.WMStatsReader import WMStatsReader

class T0DataCacheUpdate(CherryPyPeriodicTask):

    def __init__(self, rest, config):

        CherryPyPeriodicTask.__init__(self, config)

    def setConcurrentTasks(self, config):
        """
        sets the list of functions which
        """
        self.concurrentTasks = [{'func': self.gatherT0ActiveDataStats, 'duration': 300}]

    def gatherT0ActiveDataStats(self, config):
        """
        gather active data statistics
        """
        try:
            if DataCache.islatestJobDataExpired():
                wmstatsDB = WMStatsReader(config.wmstats_url, reqdbURL=config.reqmgrdb_url,
                                          reqdbCouchApp = "T0Request")
                jobData = wmstatsDB.getT0ActiveData(jobInfoFlag = True)
                DataCache.setlatestJobData(jobData)
                self.logger.info("DataCache is updated: %s", len(jobData))
        except Exception as ex:
            self.logger.error(str(ex))
        return



#!/usr/bin/env python
"""
_File_

Unittest for the WMCore.DataStructs.File class

"""

# This code written as the first real test I've written.
# -mnorman


import unittest

from Utils.PythonVersion import PY3

from WMCore.DataStructs.File import File
from WMCore.DataStructs.Run import Run


class FileTest(unittest.TestCase):
    """
    _FileTest_

    """

    def setUp(self):
        if PY3:
            self.assertItemsEqual = self.assertCountEqual

    def testDefinition(self):
        """
        This tests the definition of a DataStructs File object

        """
        testFile = File()
        self.assertEqual(testFile['lfn'], "")
        self.assertEqual(testFile['size'], 0)
        self.assertEqual(testFile['events'], 0)
        self.assertEqual(testFile['checksums'], {})
        self.assertItemsEqual(testFile['parents'], {})
        self.assertItemsEqual(testFile['locations'], {})
        self.assertFalse(testFile['merged'])

        param = {"lfn": "my_lfn",
                 "size": 1024,
                 "events": 100,
                 "checksums": {'adler32': 'BLAH', 'cksum': '12345'},
                 "parents": "my_parent",
                 "locations": {"PNN_Location"},
                 "merged": True}
        testFile = File(lfn=param['lfn'], size=param['size'], events=param['events'], checksums=param['checksums'],
                        parents=param['parents'], locations=param['locations'], merged=param['merged'])

        self.assertEqual(testFile['lfn'], param['lfn'])
        self.assertEqual(testFile['size'], param['size'])
        self.assertEqual(testFile['events'], param['events'])
        self.assertItemsEqual(testFile['checksums'], param['checksums'])
        self.assertEqual(testFile['parents'], param['parents'])
        self.assertItemsEqual(testFile['locations'], param['locations'])
        self.assertTrue(testFile['merged'])

        return

    def testAddRun(self):
        """
        This tests the addRun() function of a DataStructs File object

        """

        testLFN = "lfn"
        testSize = "1024"
        testEvents = "100"
        testCksum = "1"
        testParents = "parent"

        testLumi = 1
        testRunNumber = 1000000

        testFile = File(lfn=testLFN, size=testSize, events=testEvents, checksums=testCksum, parents=testParents)
        testRun = Run(testRunNumber, testLumi)

        testFile.addRun(testRun)

        assert testRun in testFile['runs'], "Run not added properly to run in File.addRun()"

        return

    def testComparison(self):
        """
        testComparison

        tests that File.__eq__() works properly
        ACHTUNG! File.__eq__() focuses only on self['lfn'], 
                 it does not check for any other key/property
        """

        testFile1 = File(lfn="lfn")
        testFile1_bis = File(lfn="lfn")
        self.assertEqual(testFile1, testFile1_bis)
        self.assertTrue(testFile1 == testFile1_bis)
        self.assertTrue(testFile1 is testFile1)
        self.assertTrue(testFile1 is not testFile1_bis)
        self.assertEqual(testFile1, "lfn")
        self.assertTrue(testFile1 == "lfn")
        self.assertEqual("lfn", testFile1)
        self.assertTrue("lfn" == testFile1)

        testFile2 = File(lfn="lfn-2")
        self.assertNotEqual(testFile1, testFile2)
        self.assertTrue(testFile1 != testFile2)
        self.assertNotEqual(testFile1, "lfn-2")
        self.assertTrue(testFile1 != "lfn-2")
        self.assertNotEqual("lfn-2", testFile1)
        self.assertTrue("lfn-2" != testFile1)

        # The following two comparisons work in py2 (File.File inherits __cmp__ 
        # from dict), but in py3 fail with
        # TypeError: '<' not supported between instances of 'dict' and 'dict'
        # self.assertFalse(testFile1 > testFile2)
        # self.assertTrue(testFile1 < testFile2)

        return

    def testSaveAndLoad(self):
        """
        This tests the save and load code of a DataStructs File object
        This has yet to be implemented

        """

        # It has been noted in the comments in DataStructs/File.py that
        # the save and load functions exist only to be overridden by
        # descendents of the DataStruct/File object, so I am not
        # testing this functionality.  This is just a placeholder in
        # case those requirements change.

        return

    def testSetLocation(self):
        """
        Test the `setLocation` method functionality
        """
        testFile = File(lfn="test_file")
        self.assertItemsEqual(testFile['locations'], {})

        testFile.setLocation(None)
        self.assertItemsEqual(testFile['locations'], {})

        testFile.setLocation("")
        self.assertItemsEqual(testFile['locations'], {})

        testFile.setLocation([])
        self.assertItemsEqual(testFile['locations'], {})

        testFile.setLocation("valid_PNN")
        self.assertItemsEqual(testFile['locations'], {"valid_PNN"})


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python

from __future__ import print_function

import json
import os
import unittest
from http.client import HTTPException

from WMCore_t.ReqMgr_t.TestConfig import config

from WMCore.REST.Test import fake_authz_headers
from WMCore.Services.ReqMgr.ReqMgr import ReqMgr
from WMQuality.REST.RESTBaseUnitTestWithDBBackend import RESTBaseUnitTestWithDBBackend

ADMIN = {'role': ["admin_role"], 'group': ["admin_group"]}
OPS = {'role': ["ops_role"], 'group': ["ops_group"]}
PPD = {'role': ["ppd_role"], 'group': ["ppd_group"]}


# this needs to move in better location
def insertDataToCouch(couchUrl, couchDBName, data):
    import WMCore.Database.CMSCouch
    server = WMCore.Database.CMSCouch.CouchServer(couchUrl)
    database = server.connectDatabase(couchDBName)

    doc = database.commit(data)
    return doc


def getAuthHeader(hmacData, reqAuth):
    roles = {}
    for role in reqAuth['role']:
        roles[role] = {'group': reqAuth['group']}

    return fake_authz_headers(hmacData, roles=roles, format="dict")


class ReqMgrTest(RESTBaseUnitTestWithDBBackend):
    """
    Test WorkQueue Service client
    It will start WorkQueue RESTService
    Server DB sets from environment variable.
    Client DB sets from environment variable.

    This checks whether DS call makes without error and return the results.
    Not the correctness of functions. That will be tested in different module.

    """

    def setFakeDN(self):
        # put into ReqMgr auxiliary database under "software" document scram/cmsms
        # which we'll need a little for request injection
        # Warning: this assumes the same structure in jenkins wmcore_root/test
        self.admin_header = getAuthHeader(self.test_authz_key.data, ADMIN)
        self.ops_header = getAuthHeader(self.test_authz_key.data, OPS)
        self.ppd_header = getAuthHeader(self.test_authz_key.data, PPD)

    def setUp(self, initRoot=True):
        self.setConfig(config)
        self.setCouchDBs([(config.views.data.couch_reqmgr_db, "ReqMgr"),
                          (config.views.data.couch_reqmgr_aux_db, None)])
        self.setSchemaModules([])

        super().setUp(initRoot=initRoot)

        self.setFakeDN()

        normPath = os.path.normpath(os.path.join(os.path.dirname(__file__), '..', '..', '..', '..'))
        rerecoPath = os.path.join(normPath, 'data/ReqMgr/requests/DMWM/ReReco_RunBlockWhite.json')
        with open(rerecoPath, encoding='utf-8') as jObj:
            rerecoArgs = json.load(jObj)

        self.rerecoCreateArgs = rerecoArgs["createRequest"]
        self.rerecoAssignArgs = rerecoArgs["assignRequest"]
        cmsswDoc = {"_id": "software"}
        cmsswDoc[self.rerecoCreateArgs["ScramArch"]] = []
        cmsswDoc[self.rerecoCreateArgs["ScramArch"]].append(self.rerecoCreateArgs["CMSSWVersion"])
        insertDataToCouch(os.getenv("COUCHURL"), config.views.data.couch_reqmgr_aux_db, cmsswDoc)
        self.reqSvc = ReqMgr(self.jsonSender["host"])
        self.reqSvc._noStale = True
        self.reqSvc['requests'].additionalHeaders = self.ppd_header

    def tearDown(self):
        RESTBaseUnitTestWithDBBackend.tearDown(self)

    def testRequestSimpleCycle(self):
        """
        test request cycle with one request without composite get condition.
        post, get, put
        """
        # test post method
        response = self.reqSvc.insertRequests(self.rerecoCreateArgs)
        self.assertEqual(len(response), 1)
        requestName = response[0]['request']

        ## test get method by name
        response = self.reqSvc.getRequestByNames(requestName)
        self.assertEqual(response[0][requestName]['RequestPriority'], 190000)
        self.assertEqual(len(response), 1)

        # test get by status and performing status transition
        response = self.reqSvc.getRequestByStatus('new')
        self.assertEqual(len(response), 1)

        self.reqSvc.updateRequestStatus(requestName, 'assignment-approved')
        response = self.reqSvc.getRequestByStatus('assignment-approved')
        self.assertEqual(len(response), 1)

        assignDict = {'RequestStatus': 'assigned', "AcquisitionEra": "TEST_ERA",
                      "Team": "unittest", "SiteWhitelist": ["T1_US_CBS"]}
        # PPD is not allowed to assign workflows
        with self.assertRaises(HTTPException):
            self.reqSvc.updateRequestProperty(requestName, assignDict)
        response = self.reqSvc.getRequestByStatus('assigned')
        self.assertEqual(len(response), 0)

        # trying it again with the Ops roles
        self.reqSvc['requests'].additionalHeaders = self.ops_header
        self.reqSvc.updateRequestProperty(requestName, assignDict)
        response = self.reqSvc.getRequestByStatus('assigned')
        self.assertEqual(len(response), 1)
        self.assertEqual(list(response[0].values())[0]["SiteWhitelist"], ["T1_US_CBS"])

        self.reqSvc.updateRequestStatus(requestName, 'staging')
        self.reqSvc.updateRequestStatus(requestName, 'staged')
        response = self.reqSvc.getRequestByStatus('staged')
        self.assertEqual(len(response), 1)

        self.reqSvc.updateRequestStats(requestName, {'total_jobs': 100, 'input_lumis': 100,
                                                     'input_events': 100, 'input_num_files': 100})

        # finally, make sure Ops can't perform a transition to acquired
        with self.assertRaises(HTTPException):
            self.reqSvc.updateRequestStatus(requestName, 'acquired')
        response = self.reqSvc.getRequestByStatus('acquired')
        self.assertEqual(len(response), 0)

        self.reqSvc['requests'].additionalHeaders = self.admin_header
        self.reqSvc.updateRequestStatus(requestName, 'acquired')
        response = self.reqSvc.getRequestByStatus('acquired')
        self.assertEqual(len(response), 1)

    def testRequestClone(self):
        """
        Test making a clone of a given request
        """
        # test post method
        response = self.reqSvc.insertRequests(self.rerecoCreateArgs)
        self.assertEqual(len(response), 1)
        requestName = response[0]['request']

        response = self.reqSvc.cloneRequest(requestName)
        self.assertEqual(len(response), 1)
        clonedName = response[0]['request']
        response = self.reqSvc.getRequestByNames(clonedName)
        self.assertEqual(response[0][clonedName]['TimePerEvent'], 73.85)

        response = self.reqSvc.cloneRequest(requestName, {'TimePerEvent': 20})
        self.assertEqual(len(response), 1)
        clonedName = response[0]['request']
        response = self.reqSvc.getRequestByNames(clonedName)
        self.assertEqual(response[0][clonedName]['TimePerEvent'], 20)


if __name__ == '__main__':
    unittest.main()



"""
"""
from __future__ import print_function

from future.utils import listvalues

import logging
import os
import sys
import threading

from WMCore.Database.CMSCouch import Database
from WMCore.Database.DBFormatter import DBFormatter
from WMCore.WMInit import connectToDB

def fixDBSmissingFileAssoc():
    os.environ['WMAGENT_CONFIG'] = '/data/srv/wmagent/current/config/wmagent/config.py'
    connectToDB()
    myThread = threading.currentThread()
    formatter = DBFormatter(logging, myThread.dbi)
    problemFilesSql = """
        select dbsbuffer_file.id as fileid, dbsbuffer_location.id as seid from wmbs_file_location fl
            inner join wmbs_file_details fd on fd.id = fl.fileid
            inner join wmbs_location_pnns wls on wls.location = fl.location
            inner join wmbs_location wl on wl.id = fl.location
            inner join dbsbuffer_location on dbsbuffer_location.pnn = wls.pnn
            inner join dbsbuffer_file on dbsbuffer_file.lfn = fd.lfn
            where fd.lfn in (select df.lfn from dbsbuffer_file df
                               left outer join dbsbuffer_file_location dfl on df.id = dfl.filename
                               where dfl.location is null)
                      """
    unfinishedTasks = formatter.formatDict(formatter.dbi.processData(problemFilesSql))
    print("%s lenth" % len(unfinishedTasks))
    result = {}
    for row in unfinishedTasks:
        result.setdefault(row["fileid"], row)
        print(row)
    print("trimed %s lenth" % len(result))
    insertSQL = """INSERT INTO dbsbuffer_file_location (filename, location)
               VALUES (:fileid, :seid)"""
    done = formatter.dbi.processData(insertSQL, listvalues(result))
    print("inserted %s" % done)

if __name__ == '__main__':
    fixDBSmissingFileAssoc()




#!/usr/bin/env python
"""
_SetLocation_

MySQL implementation of Files.SetLocation
"""

from WMCore.Database.DBFormatter import DBFormatter

from builtins import str, bytes

class SetLocation(DBFormatter):
    sql = """INSERT INTO wmbs_file_location (fileid, pnn)
                 SELECT :fileid, wpnn.id FROM wmbs_pnns wpnn
                 WHERE wpnn.pnn = :pnn"""

    def getBinds(self, file=None, pnn=None):
        if isinstance(pnn, (str, bytes)):
            return self.dbi.buildbinds(self.dbi.makelist(file), 'fileid',
                                       self.dbi.buildbinds(self.dbi.makelist(pnn), 'pnn'))
        elif isinstance(pnn, (list, set)):
            binds = []
            for l in pnn:
                binds.extend(self.dbi.buildbinds(self.dbi.makelist(file), 'fileid',
                                                 self.dbi.buildbinds(self.dbi.makelist(l), 'pnn')))
            return binds
        else:
            raise Exception("Type of pnn argument is not allowed: %s" \
                            % type(pnn))

    def execute(self, file, pnn, conn=None, transaction=None):
        binds = self.getBinds(file, pnn)

        result = self.dbi.processData(self.sql, binds, conn=conn,
                                      transaction=transaction)
        return



"""
Script to load example alerts data (as generated by alerts framework related
unittest) into a CouchDB database instance.

CouchDB URL and database name can be specified as command line arguments.

"""

import json
import logging
import sys
from contextlib import contextmanager
from argparse import ArgumentParser
from WMCore.Database.CMSCouch import CouchServer, Database


def _processCmdLineArgs(args):
    usage = \
        """usage: %prog options"""
    parser = ArgumentParser(usage=usage, add_help=False)
    _defineCmdLineOptions(parser)

    # opts - new processed options
    # args - remainder of the input array
    opts = parser.parse_args(args=args)
    for mandatory in ("input", "couchUrl", "database"):
        if not getattr(opts, mandatory, None):
            logging.error("Missing mandatory option ('%s')." % mandatory)
            parser.print_help()
            sys.exit(1)
    return opts.input, opts.couchUrl, opts.database


def _defineCmdLineOptions(parser):
    help = "Display this help"
    parser.add_argument("-h", "--help", help=help, action='help')
    help = "CouchDB server URL (mandatory)"
    parser.add_argument("-c", "--couchUrl", help=help)
    help = "CouchDB database name (mandatory)"
    parser.add_argument("-d", "--database", help=help)
    help = "Input data file (json format) (mandatory)"
    parser.add_argument("-i", "--input", help=help)


def _getDbConnection(couchUrl, dbName):
    """
    Check if the database exists, create if not.

    """
    couchServer = CouchServer(couchUrl)
    if not dbName in couchServer.listDatabases():
        logging.info("Database '%s' does not exits, creating it." % dbName)
        db = couchServer.createDatabase(dbName)  # returns Database
    else:
        logging.debug("Database '%s' exists." % dbName)
        db = Database(dbName, couchUrl)
    return couchServer, db


@contextmanager
def _handleFile(fileName):
    f = open(fileName, 'r')
    try:
        yield f
    finally:
        f.close()


def _processAndStoreFile(couchDb, inputFile):
    """
    Load input file.
    Each line corresponds to 1 alert (jsonized).
    Store one by one into the database.

    """
    decoder = json.decoder.JSONDecoder()
    with _handleFile(inputFile) as f:
        for alertStr in f:
            alert = decoder.decode(alertStr)
            couchDb.queue(alert)
    couchDb.commit()


def main():
    logging.basicConfig(level=logging.DEBUG)
    inputFile, couchUrl, dbName = _processCmdLineArgs(sys.argv)
    couchServer, couchDb = _getDbConnection(couchUrl, dbName)
    _processAndStoreFile(couchDb, inputFile)
    logging.info("Finished.")

    # helper stuff, checks (to remove later / if simon's happy):
    """
    allDocs = couchServer.get("/%s/_all_docs" % dbName)
    print "all documents, total_rows: %s" % allDocs["total_rows"]
    print allDocs

    # query a particular document (first) (need to know the id):
    id = allDocs["rows"][0]["id"]
    print "query document id: '%s'" % id
    db = Database(dbName, couchUrl)
    doc = db.document(id)
    print doc
    #couchServer.deleteDatabase(dbName)
    """


if __name__ == "__main__":
    main()



#!/usr/bin/env python
"""
_GetFilesForParentlessMerge_

Oracle implementation of Subscription.GetFilesForParentlessMerge
"""

from WMCore.WMBS.MySQL.Subscriptions.GetFilesForParentlessMerge import GetFilesForParentlessMerge as GetFilesForParentlessMergeMySQL

class GetFilesForParentlessMerge(GetFilesForParentlessMergeMySQL):
    pass
